#include <avr/io.h>
#include <stdlib.h>

#include "mydefs.h"
#include "usart.h"

void init(void)
{
  SPCR = _BV(SPE) | _BV(MSTR) | _BV(CPOL);
  SPSR = _BV(SPI2X);

  DDRB  = 0x2C;
  PORTB = 0x24;
}

void spiSendWord(unsigned int data)
{
  PORTB &= ~_BV(PB2);

  SPDR = (data >> 8) & 0xFF;
  while(!(SPSR & _BV(SPIF)));

  SPDR = data & 0xFF;
  while(!(SPSR & _BV(SPIF)));

  PORTB |= _BV(PB2);
}

int main(void)
{
  unsigned int low_freq = 0;
  unsigned int high_freq = 0;
  unsigned int phase = 0;
  unsigned int ctrl_reg = 0x2000;
  
  unsigned char key[32];
  unsigned char counter = 0;
  unsigned char size    = 0;
  unsigned char flag    = 1;

  usartInit(25);
  init();
  spiSendWord(0x2100);

  while(1)
  {
    while(counter != size || flag)
    {
      key[counter] = usartGetChar();

      if(counter == 0)
      {
        flag = 0;
        size = key[counter];
        counter++;
      }
      else
      {
        counter++;
      }
    }
  
    flag    = 1;
    counter = 0;
    
    switch(key[1])
    {
      case CMD_RESET:
        spiSendWord(0x2100);
        break;
      case CMD_RUN:
        spiSendWord(ctrl_reg);
        break;
      case CMD_FREQ0:
        low_freq  = (key[2] << 8) | (key[3]);
        high_freq = (key[4] << 8) | (key[5]);
        
        spiSendWord(0x4000 | low_freq);
        spiSendWord(0x4000 | high_freq);
        break;
      case CMD_FREQ1:
        low_freq  = (key[2] << 8) | (key[3]);
        high_freq = (key[4] << 8) | (key[5]);
        
        spiSendWord(0x8000 | low_freq);
        spiSendWord(0x8000 | high_freq);
        break;
      case CMD_PHASE0:
        phase = (key[2] << 8) | (key[3]);
        
        spiSendWord(0xC000 | phase);
        break;
      case CMD_PHASE1:
        phase = (key[2] << 8) | (key[3]);
        
        spiSendWord(0xE000 | phase);
        break;
      case CMD_MODE:
        ctrl_reg &= 0xFF00;
        ctrl_reg |= key[2];
        
        spiSendWord(ctrl_reg);
        break;
      case CMD_FSELECT:
        if(key[2] & 0x01)
        {
          ctrl_reg |= 0x0800;
        }
        else
        {
          ctrl_reg &= 0xF7FF;
        }
        
        spiSendWord(ctrl_reg);
        break;
      case CMD_PSELECT:
        if(key[2] & 0x01)
        {
          ctrl_reg |= 0x0400;
        }
        else
        {
          ctrl_reg &= 0xFBFF;
        }
        
        spiSendWord(ctrl_reg);
        break;
      case CMD_VERSION:
        usartPutStr(STR_VERSION);
        break;
      default:
        usartPutStr(STR_NACK);
        break;
    }
    
    usartPutStr(STR_ACK);
  }
  
  return 0;
}
