/** 
 * @file usart.h
 * @author Andreas Auer
 * @brief USART Kommunikation
 * 
 * Dieser Softwareteil dient zur Kommunikation ber die 
 * serielle Schnittstelle des Atmel Controllers. Einerseits
 * dient sie zu DEBUG Zwecken, vorallem aber findet die 
 * Software in der Kommunikation mit dem 2. Mikrocontroller
 * Verwendung!
 */

#ifndef __USART_H
#define __USART_H

/**
 * Der Funktion muss als Parameter ein Wert fr das 
 * UBRR Register bergeben werden. Durch diesen Wert
 * wird die entsprechende Baudrate eingestellt bzw. 
 * generiert!
 * @param unsigned int ubrr Parameter fr UBRR Register
 */
void usartInit(unsigned int ubrr);

/**
 * Funktion schreibt EIN Zeichen bzw. Byte auf die 
 * serielle Schnittstelle. Zuvor muss USART natrlich
 * initialisiert worden sein!
 * @param unsigned char data Byte, das auf USART geschrieben
 *               werden soll!
 */
void usartPutChar(unsigned char data);

/**
 * Gibt eine ganze Zeichenkette auf der seriellen Schnittstelle
 * aus. Schnittstelle muss ebenfalls initialisiert sein.
 * Diese Funktion ist nur fr Strings (ASCII Zeichenketten, die
 * mit '\0' abgeschlossen sind) sinnvoll, da NULL als 
 * Abbruchsbedingung fr die Funktion verwendet wird.
 * Die Funktion verwendet usartPutChar als Hilfsfunktion!
 * @param char *str Zeichenkette, die ausgegeben werden soll!
 */
void usartPutStr(char *str);

/**
 * Liest ein Zeichen von der seriellen Schnittstelle. USART muss
 * initialisiert sein!
 * @return Liefert 0, wenn keine Daten empfangen wurden, ansonsten 
 *         ungleich 0.
 */
unsigned char usartGetChar(void);

#endif
